<?php namespace App\Http\Controllers;

use Auth;
use Input;
use App\File;
use ZipArchive;
use App\Folder;

class ItemsDownloadController extends Controller {

    /**
     * Download a .zip of multiple folder and/or folders.
     *
     * @param $fileName
     * @return \Symfony\Component\HttpFoundation\BinaryFileResponse
     */
    public function downloadZip($fileName) {
        return response()->download(base_path('storage/zips/'.$fileName), '', ['Content-Type' => 'Application\Zip']);
    }

    /**
     * Download a single file by id.
     *
     * @param string|int $id
     * @return \Symfony\Component\HttpFoundation\BinaryFileResponse
     */
    public function downloadFile($id) {
        $file = File::findOrFail($id);
        return response()->download($file->getAbsolutePath(), $file->name, ['Content-Type' => $file->mime]);
    }

    /**
     * Create photos matching given share id download response.
     *
     * @param  int|string $id
     * @return string
     */
	public function createDownload() {
        $items = $this->splitFilesAndFolders();

        $downloads = [];

        foreach ($items['folders'] as $id) {
            $downloads = $this->addFolder($id, $downloads);
        }

        foreach($items['files'] as $id) {
            $file = File::find($id);
            $downloads[$this->getName($file)] = $file->getAbsolutePath();
        }

        $fileName = $this->createZip(base_path('storage/zips'), $downloads);
        return $fileName;
	}

    /**
     * Create a zip archive for download.
     *
     * @param string $path
     * @param array $downloads
     * @return string
     */
    private function createZip($path, $downloads) {
        $name = 'download-'.Auth::user()->id.'-'.date('Y-m-d').'.zip';
        $zip = new ZipArchive();

        $zip->open($path.'/'.$name, ZIPARCHIVE::OVERWRITE);

        $this->fillZip($zip, $downloads);

        $zip->close();

        return $name;
    }

    /**
     * Fill zip archive with files and folders users wants to download.
     *
     * @param ZipArchive $zip
     * @param array $downloads
     * @param string $previousFolder
     */
    private function fillZip(ZipArchive $zip, $downloads, $previousFolder = '') {
        foreach($downloads as $name => $contents) {

            //it's a folder
            if (is_array($contents)) {
                $zip->addEmptyDir($previousFolder ? $previousFolder.'/'.$name : $name);
                $this->fillZip($zip, $contents, $previousFolder ? $previousFolder.'/'.$name : $name);
            }

            //it's a file
            else {
                $zip->addFromString($previousFolder ? $previousFolder.'/'.$name : $name, file_get_contents($contents));
            }
        }
    }

    /**
     * Add given folder and its files to downloads array.
     *
     * @param int|string|Folder $folder
     * @param array $downloads
     * @return array
     */
    private function addFolder($folder, &$downloads) {
        if ( ! is_object($folder)) {
            $folder = Folder::with('files', 'folders')->find($folder);
        }

        $downloads[$folder->name] = [];

        if ( ! $folder->files->isEmpty()) {
            foreach($folder->files as $file) {
                $downloads[$folder->name][$this->getName($file)] = $file->getAbsolutePath();
            }
        }

        if ( ! $folder->folders->isEmpty()) {
            foreach($folder->folders as $subFolder) {
                $this->addFolder($subFolder, $downloads[$folder->name]);
            }
        }

        return $downloads;
    }

    /**
     * Return a file name with a proper extension.
     *
     * @param File $file
     * @return string
     */
    private function getName(File $file) {
        $extension = pathinfo($file->name, PATHINFO_EXTENSION);

        if ( ! $extension) {
            return $file->name .'.'. $file->extension;
        }

        return $file->name;
    }
}
